--
--  THE ROLLING CURSOR
--    Copyright (C) 2001 Nicholas Messenger
--    email me at:  nmessenger@spii.net
--
-- The Rolling Cursor is a really nifty effect that
-- I think would look good in a game. Take a look at
-- the demo to get acquainted with it.
--
-- GLOBAL PROCEDURES:
-- rc_init(integer color) -- call this first
-- rc_add(object task) -- call this to schedule stuff
-- rc_update() -- repeatedly call this to set the
--                display in motion
--
-- GLOBAL FUNCTION:
-- rc_busy() -- returns a boolean telling whether or not
--              all the tasks have finished
--
-- Feel free to modify, enhance, optimize, or redistribute
-- my code, as long as I am given credit for creating the
-- original version.
--
-- Note: The rolling cursor only works in
-- pixel-graphics modes with 8 by 8 text.

include graphics.e
include image.e

machine_proc(38,100) -- tick_rate(100)

constant ROLLING_CURSOR =
{{
"00011000",
"00011000",
"00011000",
"00011000",
"00011000",
"00011000",
"00011000",
"00011000"
},{
"00000110",
"00001100",
"00001100",
"00011000",
"00011000",
"00110000",
"00110000",
"01100000"
},{
"00000010",
"00000111",
"00001110",
"00011100",
"00111000",
"01110000",
"11100000",
"01000000"
},{
"00000000",
"00000001",
"00000111",
"00011110",
"01111000",
"11100000",
"10000000",
"00000000"
},{
"00000000",
"00000000",
"00000000",
"11111111",
"11111111",
"00000000",
"00000000",
"00000000"
},{
"00000000",
"10000000",
"11100000",
"01111000",
"00011110",
"00000111",
"00000001",
"00000000"
},{
"01000000",
"11100000",
"01110000",
"00111000",
"00011100",
"00001110",
"00000111",
"00000010"
},{
"01100000",
"00110000",
"00110000",
"00011000",
"00011000",
"00001100",
"00001100",
"00000110"
}} - '0'

sequence cursors   -- images for the cursor
integer cpos       -- position in the 'cursors' sequence
sequence task_list -- list of tasks for rc_update()
integer busy_flag  -- is zero when all tasks are done
atom mtimer        -- main timer
atom ctimer        -- cursor's timer

------------------------------
-- rcursor
------------------------------
procedure rcursor()
  sequence pos

  if time() > ctimer then
    ctimer += 0.05                 -- To make it go backwards:
    cpos += 1                      -- cpos -= 1
    if cpos > length(cursors) then -- if cpos < 1 then
      cpos = 1                     --   cpos = length(cursors)
    end if                         -- end if
  end if

  pos = (get_position() - 1) * 8
  display_image({pos[2],pos[1]}, cursors[cpos])
end procedure

------------------------------
-- rc_init
------------------------------
-- this *must* be called after
-- graphics_mode and before any
-- other procedure in this file
global procedure rc_init(integer color)
  text_color(color)
  cursors = ROLLING_CURSOR * color
  cpos = 1
  mtimer = -1
  task_list = {}
  busy_flag = 0
end procedure

------------------------------
-- rc_add
------------------------------
-- use this to add an item
-- to the list of tasks
global constant CLS = -1
global procedure rc_add(object task)
  task_list = append(task_list,task)
  busy_flag = 1
end procedure

------------------------------
-- rc_busy
------------------------------
-- returns whether or not the
-- task_list is empty (see demo)
global function rc_busy()
  return busy_flag
end function

------------------------------
-- rc_update
------------------------------
-- call this repeatedly to get
-- things rolling (no pun intended)
global procedure rc_update()
  integer char

  if mtimer = -1 then -- initialize the timers
    mtimer = time()
    ctimer = mtimer
  end if

  while time() > mtimer do
    if length(task_list) = 0 then -- no tasks
      busy_flag = 0
      mtimer += 1
      exit

    elsif atom(task_list[1]) then
      if task_list[1] > 0 then -- delay
        mtimer += task_list[1]
      elsif task_list[1] = CLS then -- clear-screen
        mtimer += 1
        clear_screen()
      end if
      task_list = task_list[2..length(task_list)]

    elsif length(task_list[1]) > 0 then -- string
      mtimer += 0.05
      char = task_list[1][1]
      task_list[1] = task_list[1][2..length(task_list[1])]
      if char = '\n' or char = '\r' then
        puts(1,' ') -- erases the cursor before moving down
      end if
      puts(1,char)
      rcursor()

    else -- undefined task (or empty string)
      task_list = task_list[2..length(task_list)]
    end if
  end while

  rcursor()
end procedure

